@@ -33,6 +33,7 @@ from typing import (
     List,
     Optional,
     overload,
+    Set,
     Tuple,
     Type,
     TypeVar,
@@ -48,8 +49,8 @@ except ImportError:
 HAVE_ANNOTATED = sys.version_info >= (3, 7, 0)
 
 # This is a messy, type-wise, because we not only have two potentially conflicting imports here
-# but we also define our own versions in the else block in case we operate on Python 3.5
-# or 3.6 which didn't get Annotated support in get_type_hints(). The easiest way to make mypy
+# but we also define our own versions in the else block in case we operate on Python 3.6
+# which didn't get Annotated support in get_type_hints(). The easiest way to make mypy
 # happy here is to tell it the versions from typing_extensions are canonical. Since this
 # typing_extensions import is only for mypy it'll work even without typing_extensions actually
 # installed so all's good.
@@ -77,11 +78,8 @@ else:
         return _get_type_hints(obj, globalns, localns)
 
 
-TYPING353 = hasattr(Union[str, int], '__origin__')
-
-
 __author__ = 'Alec Thomas <alec@swapoff.org>'
-__version__ = '0.18.4'
+__version__ = '0.19.0'
 __version_tag__ = ''
 
 log = logging.getLogger('injector')
@@ -236,15 +234,9 @@ class CallError(Error):
             return self.args[0]
 
         instance, method, args, kwargs, original_error, stack = self.args
-        if hasattr(method, 'im_class'):
-            instance = method.__self__
-            method_name = method.__func__.__name__
-        else:
-            method_name = method.__name__
-
         cls = instance.__class__.__name__ if instance is not None else ''
 
-        full_method = '.'.join((cls, method_name)).strip('.')
+        full_method = '.'.join((cls, method.__name__)).strip('.')
 
         parameters = ', '.join(
             itertools.chain(
@@ -365,8 +357,10 @@ class InstanceProvider(Provider):
 class ListOfProviders(Provider, Generic[T]):
     """Provide a list of instances via other Providers."""
 
+    _providers: List[Provider[T]]
+
     def __init__(self) -> None:
-        self._providers = []  # type: List[Provider[T]]
+        self._providers = []
 
     def append(self, provider: Provider[T]) -> None:
         self._providers.append(provider)
@@ -387,7 +381,7 @@ class MapBindProvider(ListOfProviders[Dict[str, T]]):
     """A provider for map bindings."""
 
     def get(self, injector: 'Injector') -> Dict[str, T]:
-        map = {}  # type: Dict[str, T]
+        map: Dict[str, T] = {}
         for provider in self._providers:
             map.update(provider.get(injector))
         return map
@@ -414,6 +408,8 @@ class Binder:
         to instantiate it on your own.
     """
 
+    _bindings: Dict[type, Binding]
+
     @private
     def __init__(self, injector: 'Injector', auto_bind: bool = True, parent: 'Binder' = None) -> None:
         """Create a new Binder.
@@ -424,7 +420,7 @@ class Binder:
         """
         self.injector = injector
         self._auto_bind = auto_bind
-        self._bindings = {}  # type: Dict[type, Binding]
+        self._bindings = {}
         self.parent = parent
 
     def bind(
@@ -522,13 +518,14 @@ class Binder:
         :param scope: Optional Scope in which to bind.
         """
         if interface not in self._bindings:
+            provider: ListOfProviders
             if (
                 isinstance(interface, dict)
                 or isinstance(interface, type)
                 and issubclass(interface, dict)
                 or _get_origin(_punch_through_alias(interface)) is dict
             ):
-                provider = MapBindProvider()  # type: ListOfProviders
+                provider = MapBindProvider()
             else:
                 provider = MultiBindProvider()
             binding = self.create_binding(interface, provider, scope)
@@ -677,47 +674,45 @@ class Binder:
         return any(_is_specialization(interface, cls) for cls in [AssistedBuilder, ProviderOf])
 
 
-if TYPING353:
+def _is_specialization(cls: type, generic_class: Any) -> bool:
+    # Starting with typing 3.5.3/Python 3.6 it is no longer necessarily true that
+    # issubclass(SomeGeneric[X], SomeGeneric) so we need some other way to
+    # determine whether a particular object is a generic class with type parameters
+    # provided. Fortunately there seems to be __origin__ attribute that's useful here.
 
-    def _is_specialization(cls: type, generic_class: Any) -> bool:
-        # Starting with typing 3.5.3/Python 3.6 it is no longer necessarily true that
-        # issubclass(SomeGeneric[X], SomeGeneric) so we need some other way to
-        # determine whether a particular object is a generic class with type parameters
-        # provided. Fortunately there seems to be __origin__ attribute that's useful here.
+    # We need to special-case Annotated as its __origin__ behaves differently than
+    # other typing generic classes. See https://github.com/python/typing/pull/635
+    # for some details.
+    if HAVE_ANNOTATED and generic_class is Annotated and isinstance(cls, _AnnotatedAlias):
+        return True
 
-        # We need to special-case Annotated as its __origin__ behaves differently than
-        # other typing generic classes. See https://github.com/python/typing/pull/635
-        # for some details.
-        if HAVE_ANNOTATED and generic_class is Annotated and isinstance(cls, _AnnotatedAlias):
-            return True
-
-        if not hasattr(cls, '__origin__'):
-            return False
-        origin = cast(Any, cls).__origin__
-        if not inspect.isclass(generic_class):
-            generic_class = type(generic_class)
-        if not inspect.isclass(origin):
-            origin = type(origin)
-        # __origin__ is generic_class is a special case to handle Union as
-        # Union cannot be used in issubclass() check (it raises an exception
-        # by design).
-        return origin is generic_class or issubclass(origin, generic_class)
-
-
-else:
-    # To maintain compatibility we fall back to an issubclass check.
-    def _is_specialization(cls: type, generic_class: Any) -> bool:
-        return isinstance(cls, type) and cls is not Any and issubclass(cls, generic_class)
+    if not hasattr(cls, '__origin__'):
+        return False
+    origin = cast(Any, cls).__origin__
+    if not inspect.isclass(generic_class):
+        generic_class = type(generic_class)
+    if not inspect.isclass(origin):
+        origin = type(origin)
+    # __origin__ is generic_class is a special case to handle Union as
+    # Union cannot be used in issubclass() check (it raises an exception
+    # by design).
+    return origin is generic_class or issubclass(origin, generic_class)
 
 
 def _punch_through_alias(type_: Any) -> type:
-    if getattr(type_, '__qualname__', '') == 'NewType.<locals>.new_type':
+    if (
+        sys.version_info < (3, 10)
+        and getattr(type_, '__qualname__', '') == 'NewType.<locals>.new_type'
+        or sys.version_info >= (3, 10)
+        and type(type_).__module__ == 'typing'
+        and type(type_).__name__ == 'NewType'
+    ):
         return type_.__supertype__
     else:
         return type_
 
 
-def _get_origin(type_: type) -> type:
+def _get_origin(type_: type) -> Optional[type]:
     origin = getattr(type_, '__origin__', None)
     # Older typing behaves differently there and stores Dict and List as origin, we need to be flexible.
     if origin is List:
@@ -795,8 +790,10 @@ class SingletonScope(Scope):
     True
     """
 
+    _context: Dict[type, Provider]
+
     def configure(self) -> None:
-        self._context = {}  # type: Dict[type, Provider]
+        self._context = {}
 
     @synchronized(lock)
     def get(self, key: Type[T], provider: Provider[T]) -> Provider[T]:
@@ -880,6 +877,9 @@ class Injector:
         ``use_annotations`` parameter is removed
     """
 
+    _stack: Tuple[Tuple[object, Callable, Tuple[Tuple[str, type], ...]], ...]
+    binder: Binder
+
     def __init__(
         self,
         modules: Union[_InstallableModuleType, Iterable[_InstallableModuleType]] = None,
@@ -888,14 +888,12 @@ class Injector:
     ) -> None:
         # Stack of keys currently being injected. Used to detect circular
         # dependencies.
-        self._stack = ()  # type: Tuple[Tuple[object, Callable, Tuple[Tuple[str, type], ...]], ...]
+        self._stack = ()
 
         self.parent = parent
 
         # Binder
-        self.binder = Binder(
-            self, auto_bind=auto_bind, parent=parent.binder if parent is not None else None
-        )  # type: Binder
+        self.binder = Binder(self, auto_bind=auto_bind, parent=parent.binder if parent is not None else None)
 
         if not modules:
             modules = []
@@ -981,15 +979,13 @@ class Injector:
                 CallError(cls, getattr(cls.__new__, '__func__', cls.__new__), (), {}, e, self._stack),
                 maximum_frames=2,
             )
+        init = cls.__init__
         try:
-            # On Python 3.5.3 calling call_with_injection() with object.__init__ would fail further
-            # down the line as get_type_hints(object.__init__) raises an exception until Python 3.5.4.
-            # And since object.__init__ doesn't do anything useful and can't have any injectable
-            # arguments we can skip calling it altogether. See GH-135 for more information.
-            if cls.__init__ is not object.__init__:
-                self.call_with_injection(cls.__init__, self_=instance, kwargs=additional_kwargs)
+            self.call_with_injection(init, self_=instance, kwargs=additional_kwargs)
         except TypeError as e:
-            reraise(e, CallError(instance, instance.__init__.__func__, (), additional_kwargs, e, self._stack))
+            # Mypy says "Cannot access "__init__" directly"
+            init_function = instance.__init__.__func__  # type: ignore
+            reraise(e, CallError(instance, init_function, (), additional_kwargs, e, self._stack))
         return instance
 
     def call_with_injection(
@@ -1066,7 +1062,7 @@ class Injector:
         try:
             for arg, interface in bindings.items():
                 try:
-                    instance = self.get(interface)  # type: Any
+                    instance: Any = self.get(interface)
                 except UnsatisfiedRequirement as e:
                     if not e.owner:
                         e = UnsatisfiedRequirement(owner_key, e.interface)
@@ -1162,7 +1158,7 @@ def get_bindings(callable: Callable) -> Dict[str, type]:
         read_and_store_bindings(
             callable, _infer_injected_bindings(callable, only_explicit_bindings=look_for_explicit_bindings)
         )
-    noninjectables = getattr(callable, '__noninjectables__', set())
+    noninjectables: Set[str] = getattr(callable, '__noninjectables__', set())
     return {k: v for k, v in cast(Any, callable).__bindings__.items() if k not in noninjectables}
 
 
@@ -1205,10 +1201,7 @@ def _infer_injected_bindings(callable: Callable, only_explicit_bindings: bool) -
             del bindings[k]
         elif _is_specialization(v, Union):
             # We don't treat Optional parameters in any special way at the moment.
-            if TYPING353:
-                union_members = v.__args__
-            else:
-                union_members = v.__union_params__
+            union_members = v.__args__
             new_members = tuple(set(union_members) - {type(None)})
             # mypy stared complaining about this line for some reason:
             #     error: Variable "new_members" is not valid as a type
@@ -1410,7 +1403,7 @@ def noninjectable(*args: str) -> Callable[[CallableT], CallableT]:
             if arg not in argspec.args and arg not in argspec.kwonlyargs:
                 raise UnknownArgument('Unable to mark unknown argument %s ' 'as non-injectable.' % arg)
 
-        existing = getattr(function, '__noninjectables__', set())
+        existing: Set[str] = getattr(function, '__noninjectables__', set())
         merged = existing | set(args)
         cast(Any, function).__noninjectables__ = merged
         return function
